#==============================================================================
# package      DE_EPAGES::AmazonPayments::UI::JSON
# state        public
# base         base DE_EPAGES::Presentation::UI::JSON
# description  JSON actions to e.g. validate SEPA data
#==============================================================================
package DE_EPAGES::AmazonPayments::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;
use JSON::XS qw ( decode_json );

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug LogError );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;


sub OrderConfirmationAmazonPayment {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;
    my $Container = $Basket->container // return;

    my $Payment = $Container->get('Payment');
    return unless ($Payment && $Payment->instanceOf('LineItemPaymentAmazonPayments'));

    my $Form = $Servlet->form;

    my $RequiredCheckboxes = decode_json($Form->value('RequiredCheckboxes'));

    LogDebug('RequiredCheckboxes', $RequiredCheckboxes);
    foreach my $RequiredCheckbox (keys %$RequiredCheckboxes ) {
        unless ($RequiredCheckboxes->{$RequiredCheckbox} eq JSON::XS::true) {
            Error('CONFIRM_FORM_VALIDATION', {'Message' => "$RequiredCheckbox not checked"});
        }
    }

    my $OptionalCheckboxes = decode_json($Form->value('OptionalCheckboxes'));
    LogDebug('OptionalCheckboxes', $OptionalCheckboxes);

    my $CustomerComment =  $Form->value('CustomerComment');
    if(defined($CustomerComment)) {
        $Basket->set({ 'CustomerComment' => $CustomerComment,
        });
    }

    my $SuccessUrl = $self->_getSuccessUrl($Servlet, $Basket, $OptionalCheckboxes);
    my $FailureUrl = $self->_getFailureUrl($Servlet, $Basket);

    eval {
        $Payment->confirmOrder($SuccessUrl, $FailureUrl);
    };
    if (ExistsError()) {
            my $Error = GetError();
            if ($Error->code() eq 'ConfirmOrderFailedRecoverable') {
                $self->_CheckAllConstraints($Payment, $Servlet);
             }else {
                $self->_handleErrors($Servlet);
            }
    }

    return;
}

sub _getSuccessUrl {
    my $self = shift;
    my ($Servlet, $Basket, $OptionalCheckboxes) = @_;

    my $hCallbackUrlOptions = {
        'Type' => 'sf/mobile',
        'AddToken' => '1',
        'UseSSL' => '1',
    };

    my $hSuccessUrlParams = {
        'ChangeAction' => 'SaveMultiCheckoutConfirmation',
    };
    foreach my $OptionalCheckbox (keys %$OptionalCheckboxes ) {
        if ( defined($OptionalCheckboxes->{$OptionalCheckbox}) && $OptionalCheckboxes->{$OptionalCheckbox} eq JSON::XS::true ) {
            $hSuccessUrlParams->{$OptionalCheckbox} = 'true';
        }
    }
    my $SuccessUrl = $self->_getCallbackShopUrl($Basket, $hSuccessUrlParams, $hCallbackUrlOptions, $Servlet);

    return $SuccessUrl;
}

sub _getFailureUrl {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

    my $hUrlParams = {
        'ViewAction' => 'ViewMultiCheckoutAmazonPayment',
    };
    my $hCallbackUrlOptions = {
        'Type' => 'sf/mobile',
        'AddToken' => '1',
        'UseSSL' => '1',
    };
    my $FailureUrl = $self->_getCallbackShopUrl($Basket, $hUrlParams, $hCallbackUrlOptions, $Servlet);

    return $FailureUrl;
}

sub _getCallbackShopUrl {
    my $self = shift;
    my ($Object, $hArgs, $hOptions, $Servlet) = @_;

    return BuildShopUrl($Object, $hArgs, $hOptions, $Servlet);
}

sub _CheckAllConstraints {
    my $self = shift;
    my ($PaymentLineItem, $Servlet) = @_;

    my $aConstraints = $PaymentLineItem->orderDetails->constraints;
    if (scalar @$aConstraints) {
        if('ShippingAddressNotSet' ~~ $aConstraints) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonAddress');
            $Servlet->form->executeFormError({
                'Name'   => 'ShippingAddressNotSet',
                'Reason' => 'ShippingAddressNotSet',
            });
        }
        if('PaymentPlanNotSet' ~~ $aConstraints) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonPayment');
            $Servlet->form->executeFormError({
                'Name'   => 'PaymentPlanNotSet',
                'Reason' => 'PaymentPlanNotSet',
            });
        }
        if('PaymentMethodNotAllowed' ~~ $aConstraints) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonPayment');
            $Servlet->form->executeFormError({
                'Name'   => 'PaymentMethodNotAllowed',
                'Reason' => 'PaymentMethodNotAllowed',
            });
        }
        if('AmountNotSet' ~~ $aConstraints) {
            $PaymentLineItem->updateOrderDetails;
        }
    }
}

sub _handleErrors {
        my $self = shift;
    my ($Servlet) = @_;

    my $Error = GetError();
    LogError($Error);

    if($Error->code() eq 'InvalidCountry') {
        $Servlet->vars( 'ViewAction','ViewMultiCheckoutAmazonAddress');
        $Servlet->form()->executeFormError({ 'Name'   => 'UnsupportedShippingAddress', 'Reason' => 'UnsupportedShippingAddress' });

    } else {
        Error('INVALID_FORM', {
            'ViewVars'=> { 'FormError' => 0, 'ViewObjectID' => $Servlet->object()->id(), 'AmazonPaymentsError' => 1 },
            'ViewAction' => 'ViewMultiCheckoutBasket'
        });
    }
    return;
}

1;