#========================================================================================
# package      DE_EPAGES::AmazonPayments::t::unit::ui::UI_JSON
# base         DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  Unit test for ExecutePayment::Service
#========================================================================================
package DE_EPAGES::AmazonPayments::t::unit::ui::JSON;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( Object HashRef );
use JSON::XS qw ( encode_json );
use strict;

use constant SHOP_URL => 'https://myshop.epages.com/';
use constant SUCCESS_URL => SHOP_URL . '?ChangeAction=SaveMultiCheckoutConfirmation&ObjectID=20845&AuthenticationStatus=Success';
use constant SUCCESS_URL_WITH_OPTIONAL_CHECKBOX =>
    SHOP_URL . '?ChangeAction=SaveMultiCheckoutConfirmation&RatingInvitationFlag=true&ObjectID=20845&AuthenticationStatus=Success';
use constant FAILURE_URL => SHOP_URL . '?ChangeAction=SaveMultiCheckoutConfirmation&ObjectID=20845&AuthenticationStatus=Failure';
use constant BASKET_ID => 20845;
use constant CUSTOMER_COMMENT => 'Some Customer Comment';

sub test {
    my $self = shift;

    $self->test_orderConfirmation_callbackurls_are_correct();
    $self->test_orderConfirmation_required_checkboxes_not_checked();
    $self->test_orderConfirmation_optional_checkboxes_checked();
    $self->test_orderConfirmation_checkboxes_checked_customer_comment_set();

    done_testing();
}

sub test_orderConfirmation_callbackurls_are_correct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LineItemPayment = $self->_createLineItemPayment({});
    my $LineItemContainer = $self->_createLineItemContainer({
        'Payment' => $LineItemPayment,
    });
    my $Basket = $self->_createBasket({'LineItemContainer' => $LineItemContainer});
    my $withRequiredCheckboxes = 1;
    my $withOptionalCheckboxes = 0;
    my $Servlet = $self->_createServlet($Basket, $withRequiredCheckboxes, $withOptionalCheckboxes);

    my $SUT = $self->_createSUT($Basket);

    $SUT->OrderConfirmationAmazonPayment($Servlet);

    my $aConfirmOrderParameters = GetParametersFromMockifyCall($LineItemPayment , 'confirmOrder');
    my $aExpectedParameters = [SUCCESS_URL, FAILURE_URL];

    is_deeply($aConfirmOrderParameters, $aExpectedParameters, "$SubTestName - Test if SuccessUrl and FailureUrl are correctly setup");

}


sub test_orderConfirmation_required_checkboxes_not_checked {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LineItemPayment = $self->_createLineItemPayment({});
    my $LineItemContainer = $self->_createLineItemContainer({
        'Payment' => $LineItemPayment,
    });
    my $Basket = $self->_createBasket({'LineItemContainer' => $LineItemContainer});
    my $withRequiredCheckboxes = 0;
    my $withOptionalCheckboxes = 0;
    my $Servlet = $self->_createServlet($Basket, $withRequiredCheckboxes, $withOptionalCheckboxes);

    my $SUT = $self->_createSUT($Basket);

    throws_ok( sub {$SUT->OrderConfirmationAmazonPayment($Servlet)},
        qr/CONFIRM_FORM_VALIDATION/,
        "$SubTestName with isValidCheckbox = 0 --> got error response");

}

sub test_orderConfirmation_optional_checkboxes_checked {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LineItemPayment = $self->_createLineItemPayment({});
    my $LineItemContainer = $self->_createLineItemContainer({
        'Payment' => $LineItemPayment,
    });
    my $Basket = $self->_createBasket({'LineItemContainer' => $LineItemContainer});
    my $withRequiredCheckboxes = 1;
    my $withOptionalCheckboxes = 1;
    my $Servlet = $self->_createServlet($Basket, $withRequiredCheckboxes, $withOptionalCheckboxes);

    my $SUT = $self->_createSUT($Basket);

    $SUT->OrderConfirmationAmazonPayment($Servlet);

    my $aConfirmOrderParameters = GetParametersFromMockifyCall($LineItemPayment , 'confirmOrder');

    is_deeply($aConfirmOrderParameters->[0], SUCCESS_URL_WITH_OPTIONAL_CHECKBOX, "$SubTestName - Test SuccessUrl with optional checkbox set");

}

sub test_orderConfirmation_checkboxes_checked_customer_comment_set {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $LineItemPayment = $self->_createLineItemPayment({});
    my $LineItemContainer = $self->_createLineItemContainer({
        'Payment' => $LineItemPayment,
    });
    my $Basket = $self->_createBasket({'LineItemContainer' => $LineItemContainer});
    my $withRequiredCheckboxes = 1;
    my $withOptionalCheckboxes = 1;
    my $withCustomerComment = CUSTOMER_COMMENT;
    my $Servlet = $self->_createServlet($Basket, $withRequiredCheckboxes, $withOptionalCheckboxes, $withCustomerComment);

    my $SUT = $self->_createSUT($Basket);

    $SUT->OrderConfirmationAmazonPayment($Servlet);

    my $aConfirmOrderParameters = GetParametersFromMockifyCall($LineItemPayment , 'confirmOrder');

    is_deeply($aConfirmOrderParameters->[0], SUCCESS_URL_WITH_OPTIONAL_CHECKBOX, "$SubTestName - Test SuccessUrl with optional checkbox set");

    my $BasketSaved = $Servlet->object;
    my $SavedComment = $BasketSaved->get('CustomerComment');
    ok( $SavedComment, "$SubTestName - Customer comment is set.");
    is( $SavedComment, CUSTOMER_COMMENT, "$SubTestName - Customer comment has expected value.");

}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Basket) = @_;

    my $MockifySut = Test::Mockify::Sut->new(
                   'DE_EPAGES::AmazonPayments::UI::JSON',
                   []
    );

    my $hCallbackUrlOptions = {
        'Type' => 'sf/mobile',
        'AddToken' => '1',
        'UseSSL' => '1',
    };
    my $hSuccessUrlParams = {
        'ChangeAction' => 'SaveMultiCheckoutConfirmation',
    };
    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
                ->when(Object('DE_EPAGES::Order::API::Object::Basket'),
                        HashRef($hSuccessUrlParams), HashRef($hCallbackUrlOptions),
                        Object('DE_EPAGES::Presentation::Servlet::Permission'))
                ->thenReturn($self->_createAmazonResponseUrl($Basket, $hSuccessUrlParams) . '&AuthenticationStatus=Success');

    my $hSuccessUrlParamsWithOptionaCheckbox = {
        'ChangeAction' => 'SaveMultiCheckoutConfirmation',
        'RatingInvitationFlag' => 'true'
    };
    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
            ->when(Object('DE_EPAGES::Order::API::Object::Basket'),
                    HashRef($hSuccessUrlParamsWithOptionaCheckbox), HashRef($hCallbackUrlOptions),
                    Object('DE_EPAGES::Presentation::Servlet::Permission'))
            ->thenReturn($self->_createAmazonResponseUrl($Basket, $hSuccessUrlParamsWithOptionaCheckbox) . '&AuthenticationStatus=Success');

    my $hFailureUrlParams = {
        'ViewAction' => 'ViewMultiCheckoutAmazonPayment',
    };
    $MockifySut->mockImported('DE_EPAGES::Shop::API::Url', 'BuildShopUrl')
                ->when(Object('DE_EPAGES::Order::API::Object::Basket'),
                        HashRef($hFailureUrlParams), HashRef($hCallbackUrlOptions),
                        Object('DE_EPAGES::Presentation::Servlet::Permission'))
                ->thenReturn($self->_createAmazonResponseUrl($Basket, $hSuccessUrlParams) . '&AuthenticationStatus=Failure');

    return $MockifySut->getMockObject();
}

sub _createBasket {
    my $self = shift;
    my ($hAttributes) = @_;

    $hAttributes->{'ID'} = BASKET_ID;
    $hAttributes->{'CustomerComment'} = undef;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createLineItemContainer {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createLineItemPayment {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::AmazonPayments::API::Object::LineItemPaymentAmazonPayments',
        $hAttributes
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('instanceOf', 1, ['string']);
    $Mockify->addMockWithReturnValueAndParameterCheck('confirmOrder', 1, ['string', 'string']);

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Basket, $withRequiredCheckboxes, $withOptionalCheckboxes, $withCustomerComment) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Basket);
    $MockifyServlet->addMockWithReturnValue(
            'form',
            $self->_createForm($withRequiredCheckboxes, $withOptionalCheckboxes, $withCustomerComment)
    );
    return $MockifyServlet->getMockObject();
}

sub _createForm {
    my $self = shift;
    my ($withRequiredCheckboxes, $withOptionalCheckboxes, $withCustomerComment) = @_;

    my $JsonRequiredCheckboxes = encode_json({
            'TermsAndConditionsFlag' => $withRequiredCheckboxes ? 'true' : 'false',
            'CustomerInformationFlag' => $withRequiredCheckboxes ? 'true' : 'false'
    });

    my $JsonNoCheckboxes = encode_json({});
    my $JsonOptionalCheckboxes = $JsonNoCheckboxes;

    if ($withOptionalCheckboxes) {
       $JsonOptionalCheckboxes = encode_json({
            'RatingInvitationFlag' => 'true'
        });
    }

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::WebInterface::API::Form');
    $MockedForm->mock(
        'value',
        sub {
            shift;
            my ($FormValue) = @_;
            if ($FormValue eq 'RequiredCheckboxes') {
                return $JsonRequiredCheckboxes;
            } elsif ($FormValue eq 'OptionalCheckboxes') {
                return $JsonOptionalCheckboxes;
            } elsif ($FormValue eq 'CustomerComment') {
                return $withCustomerComment;
            } else {
                return $JsonNoCheckboxes;
            }
        }
    );

    return $MockedForm;
}

sub _createAmazonResponseUrl {
    my $self = shift;
    my ($Basket, $hUrlParams) = @_;
    
    my $QueryParams = '';
    for my $Parameter (keys %$hUrlParams) {
        if ($Parameter eq 'ChangeAction' || $Parameter eq 'ViewAction') {
            if (length($QueryParams) > 0) {
                $QueryParams = $Parameter . '=' . $hUrlParams->{$Parameter} . '&' . $QueryParams;
            } else {
                $QueryParams = $Parameter . '=' . $hUrlParams->{$Parameter};
            }
        } else {
            if (length($QueryParams) > 0) {
                $QueryParams = $QueryParams . '&' . $Parameter . '=' . $hUrlParams->{$Parameter};
            } else {
                $QueryParams = $Parameter . '=' . $hUrlParams->{$Parameter};
            }
        }
    }
    
    return SHOP_URL . '?' . $QueryParams . '&' . 'ObjectID=' . $Basket->id();
}

__PACKAGE__->runScript();