#========================================================================================
# package      DE_EPAGES::Core::API::Object::DateTimeFormatter
# state        public
#----------------------------------------------------------------------------------------
# description  Formatter and parser for locale-dependent time and date strings. Based on
#               DateTime::Format::Strptime.
#----------------------------------------------------------------------------------------
# example      my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(
#                   locale => 'de_DE',
#               );
#               my $Date   = $Formatter->parse_datetime( 'Donnerstag, 7. Oktober 2004', 'date' );
#               my $String = $Formatter->format_datetime( $Date, 'date', 'long' );
#========================================================================================
package DE_EPAGES::Core::API::Object::DateTimeFormatter;
use base DateTime::Format::Strptime;

use strict;
use DateTime::Locale;
use DE_EPAGES::Core::API::Error qw (CallerError Error ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw (GetLocalTimeZone GetLocalLocale GetCurrentDateTime IsA);
use DE_EPAGES::Core::API::TestParameter qw ( TestString );

my %FORMAT_PARSE_MAP = (
    '%{hour}'                => '%H',
    '%{time_zone_long_name}' => '%z',
);

#ERRORDEFINITION
sub ErrorDefinition { return {
    'FORMAT_NOT_DATETIME' => 'value is no #Type (#Value) (req. format: #Format)',
    'FORMAT_UNKNOWN' => 'unknown format: #Type (allowed formats are: #Allowed)',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a formatter for dates and/or time strings.
#----------------------------------------------------------------------------------------
# input        %options | keys
#               <ul>
#                   <li>locale       - (optional) locale id, example: 'en_US',
#                                                 default: server locale - string</li>
#                   <li>time_zone    - (optional) time zone, example: '+01:00',
#                                                 default: server time zone - string</li>
#               </ul>| hash
# return       $Formatter | formatter object | object
#========================================================================================
sub new {
    my $class = shift;
    my (%options) = @_;
    $options{'time_zone'} = GetLocalTimeZone() unless defined $options{'time_zone'};
    $options{'locale'}    = GetLocalLocale()   unless defined $options{'locale'};
    $options{'pattern'}   = '%x %X';
    return bless DateTime::Format::Strptime->new(%options), $class;
}

#========================================================================================
# function     formatString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormatString = $Formatter->formatString($type, $formattype);
#----------------------------------------------------------------------------------------
# description  returns formats string for parse_datetime and format_datetime
#----------------------------------------------------------------------------------------
# input        $type | date/time/datetime/ | string
# input        $formattype | short/medium/long/full (optional short)| string
# return       $FormatString | pattern for DateTime::Format::Strptime | string
#========================================================================================
sub formatString {
    my $self = shift;
    my ($type, $formattype) = @_;
    TestString('type',$type);

    if ($type !~ /^(?:date|time|datetime)$/) {
        CallerError('FORMAT_UNKNOWN', {
            'Type'   => $type,
            'Allowed' => 'date, datetime or time'
        })
    }

    $formattype = 'short' unless defined $formattype;

    my $functionname = $type.'_format_'.$formattype;
    my $locale = DateTime::Locale->load($self->locale);
    my $formatString = $locale->_cldr_to_strftime($locale->$functionname()); ## no critic qw(ProhibitPrivateSubroutineUse)
    $formatString =~ s/\Q$_\E/$FORMAT_PARSE_MAP{$_}/ foreach keys %FORMAT_PARSE_MAP;

    if ($self->locale eq 'da_DK') {
        $formatString =~ s/[\/]/-/g;
        $formatString =~ s/[.]/:/g;
    }

    # always use 4 digits for year
    $formatString =~ s/%y/%{ce_year}/g;

    return $formatString;
}

#========================================================================================
# function     format_datetime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FormattedDateTime = $Formatter->format_datetime($value, $type, $formattype);
#----------------------------------------------------------------------------------------
# description  Formats the value
#----------------------------------------------------------------------------------------
# input        $value | datetime | object
# input        $type | date/time/datetime | string
# input        $formattype | (optional) short/medium/long/full
#                               or custom strftime format, for example '%Y-%m-%d'
#                               (default: short)| string
# return       $FormattedDateTime | formatted value | string
#========================================================================================
sub format_datetime {
    my $self = shift;
    my ($value, $type, $formattype) = @_;
    die 'no DateTime object' unless IsA( $value, 'DateTime' );
    TestString('type',$type);

    if ($type !~ /^(?:date|time|datetime)$/) {
        CallerError('FORMAT_UNKNOWN', {
            'Type'   => $type,
            'Allowed' => 'date, datetime or time'
        })
    }

    my $Locale = $self->locale;
    my $OldLocale = $self->locale;

    # use Bokmal instead of Nynorsk
    $Locale = 'nb_NO' if $Locale eq 'no_NO';

    # swiss exception
    if($OldLocale =~ /^en_CH|it_CH$/){
        $Locale = 'de_CH';
        $self->{locale} = $Locale;
    }

    $value->set(locale => $Locale);

    my $formatString;
    if( not defined $formattype or $formattype =~ /^(short|medium|long|full)$/x ) {
        $formatString = $self->formatString($type, $formattype)
    }
    else {
        $formatString = $formattype;
    }

    # end swiss exception
    if($OldLocale =~ /^en_CH|it_CH$/){
        $value->set(locale => $OldLocale);
        $self->{locale} = $OldLocale
    }

    return $value->strftime( $formatString );
}

#========================================================================================
# function     parse_datetime
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $Formatter->parse_datetime($FormattedDateTime, $type, $formattype);
#----------------------------------------------------------------------------------------
# description  Return the internal value of a formatted value.
#----------------------------------------------------------------------------------------
# input        $FormattedDateTime | formatted value | string
# input        $type | date/time/datetime | string
# input        $formattype | (optional) short/medium/long/full
#                               or custom strftime format, for example '%Y-%m-%d'
#                               (default: short)| string
# return       $value | datetime value | DateTime object
#========================================================================================
sub parse_datetime {
    my $self = shift;
    my ($string, $type, $formattype) = @_;
    TestString('type', $type);

    if ($type !~ /^(?:date|time|datetime)$/) {
        CallerError('FORMAT_UNKNOWN', {
            'Type'   => $type,
            'Allowed' => 'date, datetime or time'
        })
    }

    # swiss exception
    my $OldLocale = $self->locale;
    if($OldLocale =~ /^en_CH|it_CH$/){
        $self->{locale} = 'de_CH';
    }

    # strip leading and trailing spaces
    $string =~ s/^\s*(.*?)\s*$/$1/x;

    my $formatString  = ( !defined $formattype || $formattype =~ /^(short|medium|long|full)$/x )
        ? $self->formatString($type, $formattype)
        : $formattype;

    # end swiss exception
    if($OldLocale =~ /^en_CH|it_CH$/){
        $self->{locale} = $OldLocale;
    }

    # fix for EPG-6933
    # reason: if the pattern contains time zone information like "+0200" and there is no date
    # information available, the parser binds the given time to 0001-01-01 (YES!, 1 a.d.). Since
    # there was no unique calendar, the time is calculated based on sun rise and may differ
    # various minutes or hours.
    if ($type eq 'time' && $formatString =~ /%z/) {
        my $Now = GetCurrentDateTime;
        $formatString = '%Y-%m-%d ' . $formatString;
        # use today (e.g. for daylight saving conditions)
        $string = $Now->ymd('-') . ' ' . $string;
    }

    $self->pattern($formatString);
    my $value = eval { return $self->SUPER::parse_datetime($string); };
    if (ExistsError()) {
        Error( GetError() );
    }
    elsif (not defined $value) {
        Error('FORMAT_NOT_DATETIME', {
            'Type'   => $type,
            'Value'  => $string,
            'Format' => $formatString,
            'Locale' => $self->locale,
            'Example' => $self->format_datetime( DateTime->now, $type),
        })
    }
    return $value;
}

1;
