#========================================================================================
# package      DE_EPAGES::Ekomi::API::TLE::ShopHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Implements TLE functions for shop.
#========================================================================================
package DE_EPAGES::Ekomi::API::TLE::ShopHandler;

use strict;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# input        %args | args | array
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless \%args, $class;
    return $self;
}

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    $Processor->registerHandler( 'FunctionHandler', $self, 'ShopWithEkomiConfiguration' );
    return;
}

#========================================================================================
# function     ShopWithEkomiConfiguration
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ShopWithEkomiConfiguration(#ShopObject)
#----------------------------------------------------------------------------------------
# example      #IF( #ShopWithEkomiConfiguration(#Shop.Object) EQ "1" ) ... #ENDIF
#----------------------------------------------------------------------------------------
# description  Informs about Ekomi Seal of Approval to be configured (for deprecation purposes)
#----------------------------------------------------------------------------------------
# input        #ShopObject | shop object | object
# return       #ShopHasEkomiConfiguration | shop has Ekomi configuration | boolean
#========================================================================================
sub ShopWithEkomiConfiguration {
    my $self = shift;
    my ( $Processor, $aParams ) = @_;

    my ($Shop) = @{$aParams};

    TestObject('Shop', $Shop);
    my $ShopClass = $Shop->class();
    return 0 unless($ShopClass->existsAttribute('eKomiInterfaceID'));
    return 0 unless($ShopClass->existsAttribute('eKomiInterfacePW'));

    my $HasEkomiInterfaces = 0;
    foreach my $LanguageID (map {$_->{'LanguageID'}} @{ $Shop->get('Languages') }) {
        my $eKomiInterfaceID = $Shop->get('eKomiInterfaceID', $LanguageID);
        next unless(defined $eKomiInterfaceID);
        my $eKomiInterfacePW = $Shop->get('eKomiInterfacePW', $LanguageID);
        next unless(defined $eKomiInterfacePW);
        $HasEkomiInterfaces = 1;
        last;
    }

    return $HasEkomiInterfaces;
}
1;