#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use Test::Mockify;
use Test::Mockify::Verify qw ( WasCalled );
use Test::Mockify::Matcher qw ( Object String );
use Test::Mockify::Sut;
use DE_EPAGES::MolliePayments::API::Constants qw (
    PAYMENT_STATUS_PAID
    PAYMENT_STATUS_FAILED
);

use strict;

use constant SHOP_ALIAS => 'MyShop';
use constant LINE_ITEM_PAYMENT_GUID => 'AJBD-IOHH-435-IUGU';
use constant MOLLIE_ID => 'tr_d0b0E3EA3v';
use constant OBJECT_ID => '6541';
use constant DEREFERRER_TOKEN => 'DEREFERRER_TOKEN';
use constant TOKEN_PAYLOAD => sprintf('%s&%s&%s', SHOP_ALIAS, OBJECT_ID, LINE_ITEM_PAYMENT_GUID);
use constant TRANS_TIME => '2120-23-04 34:98:64';

sub test {
    my $self = shift;

    $self->test_MolliePaymentsPaymentWebhook_status_paid();
    $self->test_MolliePaymentsPaymentWebhook_status_failed();
    $self->test_MolliePaymentsPaymentWebhook_status_paid_order();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_MolliePaymentsPaymentWebhook_status_paid {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopAttributes = {
        'Alias' => SHOP_ALIAS
    };

    my $MockedShop = $self->_createShop($hShopAttributes);

    my $hServletParams = {
        'id' => MOLLIE_ID,
        'ObjectID' => OBJECT_ID,
        'LineItemPaymentGUID' => LINE_ITEM_PAYMENT_GUID,
        'DereferrerToken' => DEREFERRER_TOKEN
    };
    my $MockedServlet = $self->_createServlet($MockedShop, $hServletParams);
    my $MockedPaymentStatusService = $self->_createPaymentStatusService(PAYMENT_STATUS_PAID);
    my $Order = $self->_createOrder();

    my $SUT = $self->_createSUT($MockedPaymentStatusService, $Order);

    lives_ok(
        sub { $SUT->MolliePaymentsPaymentWebhook($MockedServlet);},
        $SubTestName . ' - test if MolliePaymentsPaymentWebhook can be called without errors'
    );

    ok( 
        WasCalled($MockedPaymentStatusService, 'handleStatusOnPaymentWebhook'),
        $SubTestName . ' - Verify that handleStatusOnPaymentWebhook was correctly called'
    );
    ok( 
        WasCalled($SUT, 'DE_EPAGES::Order::UI::Basket::doBasket2Order'),
        $SubTestName . ' - Verify that doBasket2Order was correctly called'
    );
    is( 
        $Order->get('PaidOn'),
        TRANS_TIME,
        $SubTestName . ' - Verify that Order was correct PaidOn'
    );
    ok( 
        WasCalled($MockedPaymentStatusService, 'updateMollieOrderName'),
        $SubTestName . ' - Verify that updateMollieOrderName was correctly called'
    );
    return;
}

sub test_MolliePaymentsPaymentWebhook_status_failed {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopAttributes = {
        'Alias' => SHOP_ALIAS
    };

    my $MockedShop = $self->_createShop($hShopAttributes);

    my $hServletParams = {
        'id' => MOLLIE_ID,
        'ObjectID' => OBJECT_ID,
        'LineItemPaymentGUID' => LINE_ITEM_PAYMENT_GUID,
        'DereferrerToken' => DEREFERRER_TOKEN
    };
    my $MockedServlet = $self->_createServlet($MockedShop, $hServletParams);
    my $MockedPaymentStatusService = $self->_createPaymentStatusService(PAYMENT_STATUS_FAILED);
    my $Order = $self->_createOrder();

    my $SUT = $self->_createSUT($MockedPaymentStatusService, $Order);

    lives_ok(
        sub { $SUT->MolliePaymentsPaymentWebhook($MockedServlet);},
        $SubTestName . ' - test if MolliePaymentsPaymentWebhook can be called without errors'
    );

    ok(
        WasCalled($MockedPaymentStatusService, 'handleStatusOnPaymentWebhook'),
        $SubTestName . ' - Verify that handleStatusOnPaymentWebhook was correctly called'
    );
    ok(
        !WasCalled($SUT, 'DE_EPAGES::Order::UI::Basket::doBasket2Order'),
        $SubTestName . ' - Verify that doBasket2Order was not called'
    );
    is(
        $Order->get('PaidOn'),
        undef,
        $SubTestName . ' - Verify that Order has undef PaidOn'
    );
    ok( 
        !WasCalled($MockedPaymentStatusService, 'updateMollieOrderName'),
        $SubTestName . ' - Verify that updateMollieOrderName was not called'
    );
    return;
}

sub test_MolliePaymentsPaymentWebhook_status_paid_order {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hShopAttributes = {
        'Alias' => SHOP_ALIAS
    };

    my $MockedShop = $self->_createShop($hShopAttributes);

    my $hServletParams = {
        'id' => MOLLIE_ID,
        'ObjectID' => OBJECT_ID,
        'LineItemPaymentGUID' => LINE_ITEM_PAYMENT_GUID,
        'DereferrerToken' => DEREFERRER_TOKEN
    };
    my $MockedServlet = $self->_createServlet($MockedShop, $hServletParams);
    my $MockedPaymentStatusService = $self->_createPaymentStatusService(PAYMENT_STATUS_PAID);
    my $Order = $self->_createOrder();

    my $SUT = $self->_createSUT($MockedPaymentStatusService, $Order, my $IsPaymentOfOrder = 1);

    lives_ok(
        sub { $SUT->MolliePaymentsPaymentWebhook($MockedServlet);},
        $SubTestName . ' - test if MolliePaymentsPaymentWebhook can be called without errors'
    );

    ok( 
        WasCalled($MockedPaymentStatusService, 'handleStatusOnPaymentWebhook'),
        $SubTestName . ' - Verify that handleStatusOnPaymentWebhook was correctly called'
    );
    ok(
        !WasCalled($SUT, 'DE_EPAGES::Order::UI::Basket::doBasket2Order'),
        $SubTestName . ' - Verify that doBasket2Order was not called'
    );
    is(
        $Order->get('PaidOn'),
        undef,
        $SubTestName . ' - Verify that Order has undef PaidOn'
    );
    ok( 
        !WasCalled($MockedPaymentStatusService, 'updateMollieOrderName'),
        $SubTestName . ' - Verify that updateMollieOrderName was not called'
    );
    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createServlet {
    my $self = shift;
    my ($MockedShop, $hServletParams) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $MockedShop );

    $Mockify->addParams($hServletParams);
    $Mockify->addMockWithReturnValueAndParameterCheck('writeContentOutput', 1, [{'string' => 'OK'}]);

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createSUT {
    my $self = shift;
    my ($PaymentStatusService, $Order, $IsPaymentOfOrder) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::UI::Shop');

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadObjectByGUID')->when(String(LINE_ITEM_PAYMENT_GUID))->thenReturn($self->_createLineItemPaymentMolliePayments($IsPaymentOfOrder));
    $MockifySut->mockStatic('DE_EPAGES::Order::UI::Basket::doBasket2Order')->when(String(), Object(), Object())->thenReturn($Order);
    $MockifySut->mock('_ValidateDereferrerToken')->when(String(DEREFERRER_TOKEN), String(TOKEN_PAYLOAD))->thenReturn(1);
    $MockifySut->mock('_createPaymentStatusService')->when()->thenReturn($PaymentStatusService);
    $MockifySut->mock('_LogPayment')->whenAny()->thenReturn(1);

    return $MockifySut->getMockObject();
}

sub _createPaymentStatusService {
    my $self = shift;
    my ($Status) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::PaymentStatusService');
    $Mockify->mock('handleStatusOnPaymentWebhook')->when(Object(), String(MOLLIE_ID))->thenReturn($Status);
    $Mockify->mock('updateMollieOrderName')->when(Object('DE_EPAGES::Order::API::Object::CustomerOrder'), Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'))->thenReturnUndef();

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'ID' => OBJECT_ID,
        'instanceOf' => 'CustomerOrder',
        'PaidOn' => undef
    };
    my $MockifyOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyOrder->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;

    my $hAttributes = {
        'ID' => OBJECT_ID,
        'instanceOf' => 'Basket',
    };
    my $MockifyOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::Basket', $hAttributes
    );

    return $MockifyOrder->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($IsBasket) = @_;

    my $hAttributes = {};
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );
    $MockifyContainer->addMockWithReturnValue('parent', ($IsBasket ? $self->_createOrder() : $self->_createBasket()));

    return $MockifyContainer->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemPaymentMolliePayments {
    my $self = shift;
    my ($IsBasket) = @_;

    my $hAttributes = {
        'TransTime' => TRANS_TIME,
        'LineItemContainer' => $self->_createLineItemContainer($IsBasket),
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();
