#========================================================================================
# package      DE_EPAGES::Order::Hooks::Order
# state        public
# description  Additional hook function for OrderCreated hook.
#========================================================================================
package DE_EPAGES::Order::Hooks::Order;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError ResetError );
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Order::API::Order qw (SendOrderConfirmationMail);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );

#========================================================================================
# function     OnOrderCreatedConfirmationMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreatedConfirmationMail( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by hook API_OrderCreated.
#               Sends the order confirmation e-mail to the customer if the option
#               $hParams->{Options}->{SendConfirmationMail}.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Order - order object</li>
#                   <li>Options - options hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderCreatedConfirmationMail {
    my ($hParams) = @_;

    return unless $hParams->{Options}->{SendConfirmationMail};
    my $Order = $hParams->{'Order'};

    my $MailType = GetShopMailType( $Order->getSite(), 'CustomerOrderConfirmation', 1 );
    return if not defined $MailType;

    eval {SendOrderConfirmationMail($hParams->{'Order'});};

    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);

        Warning('SEND_MAIL_FAILED', {'Code' => $Error->code,
                                     'Message' => $Error->message,
                                    });
    }

    return ;
}

#========================================================================================
# function     OnOrderExecutePayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderExecutePayment( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by hook UI_OrderCreated to interrupt order process to give
#               the user the possiblity to go to payment server.
#               The payment method has to handle if it has already all data.
#               Attention: everything else should be handled before!
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Basket
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderExecutePayment {
    my ($hParams) = @_;
    my $Order = $hParams->{'Order'};

    my $PaymentLineItem = $Order->get('LineItemContainer')->get('Payment');
    return unless $PaymentLineItem;
    my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
    my $DoPaymentOnBasket = $PaymentMethod->get('DoPaymentOnBasket');
    $PaymentLineItem->executePayment($hParams) unless $DoPaymentOnBasket;

    return;
}

#========================================================================================
# function     OnOrderCreatedSetOrderStatusPaidOn
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreatedSetOrderStatusPaidOn( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by hook API_OrderCreated.
#               Set the status PaidOn to the current date time if
#               total amount of order is 0.00
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Order - order object</li>
#                   <li>Options - options hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderCreatedSetOrderStatusPaidOn {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};

    # test if we can mark order as paid
    if( ($Order->container->get('PaymentSubTotal') <= 0) and
        ($Order->container->get('GrandTotal')      <= 0) ) {

        my $now = GetCurrentDBHandle()->currentDateTime;
        $Order->set({'PaidOn' => $now});

    }

    return ;
}

#========================================================================================
# function     OnAfterUpdateCustomerOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateCustomerOrder( $hValues );
#----------------------------------------------------------------------------------------
# description  Checks the status of the the order notifications.
#               (Hook OBJ_AfterUpdateCustomerOrder)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateCustomerOrder {
    my ($hParams) = @_;

    my $Order = $hParams->{'Object'};
    # we do not change the notification for editable orders
    return if $Order->get('Parent')->alias eq 'EditableOrders';

    my $hNewValues = $hParams->{'Values'};
    my $Shop = $Order->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        # change the status of the product notification for Unity shops
        if (exists $hNewValues->{'ViewedOn'}) {
            my $NewOrdersNotification = $Shop->folder('ShopNotifications')->child('NewOrders');
            my $NewViewedOn = $hNewValues->{'ViewedOn'};
            # order is unseen
            if (!defined $NewViewedOn) {
                $NewOrdersNotification->addObject($Order);
            # order is seen
            } else {
                $NewOrdersNotification->deleteObject($Order);
            }
        }
    }
}

#========================================================================================
# function     OnDeleteCustomerOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteCustomerOrder( $hValues );
#----------------------------------------------------------------------------------------
# description  deletes the NewOrders notifications
#               (Hook OBJ_DeleteCustomerOrder)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Options - search options - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteCustomerOrder {
    my ($hParams) = @_;

    my $Order = $hParams->{'Object'};

    my $Shop = $Order->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        my $NewOrdersNotification = $Shop->folder('ShopNotifications')->child('NewOrders');
        # change the status of the NewOrders notification for Unity shops
        $NewOrdersNotification->deleteObject($Order);
    }
}

#========================================================================================
# function     OnOrderCreatedShopNotification
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreatedShopNotification( $hValues );
#----------------------------------------------------------------------------------------
# description  activates the NewOrders notifications
#               (Hook API_OrderCreated)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Order - order object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnOrderCreatedShopNotification {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};
    # we do not change the notification for editable orders
    return if $Order->get('Parent')->alias eq 'EditableOrders';

    my $hValues = $hParams->{'Values'};
    my $Shop = $Order->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        my $NewViewedOn = $hValues->{'ViewedOn'};
        # change the status of the new orders notification for Unity shops
        if (!defined $NewViewedOn) {
            my $NewOrdersNotification = $Shop->folder('ShopNotifications')->child('NewOrders');

            # create notification, try it again if we have an error
            my $tryit = 1;
            my $trycount = 5;
            while ($tryit) {
                $tryit = 0;
                $trycount--;
                eval {
                    $NewOrdersNotification->addObject($Order);
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    Error($Error) unless $Error->code eq 'DBIExecute' and $Error->vars->{'code'} == 1062 and $trycount; ## no critic qw(ProhibitMagicNumbers) # double key
                    $tryit = 1;
                }
            }
        }
    }
}

#========================================================================================
# function     OnBeforeEditOrder2Order
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeEditOrder2Order( $hValues );
#----------------------------------------------------------------------------------------
# description  adds the editable order to the NewOrders notifications
#               if an editable order was saved and used as order
#               and if the original order has not ViewedOn date
#               (Hook BeforeEditOrder2Order)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                  <li>Order - order object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeEditOrder2Order {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};
    my $Shop = $Order->getSite();

    return unless $Shop->get('IsUnityRegistered');

    # test if the original order is in the list of order notifications
    # then remove the original order from notifications and add the copied order
    # because we change only the parent for the copy and delete the original order
    # so the notification objects must be changed
    unless($Order->get('ViewedOn')) {
        my $NewOrdersNotification = $Shop->folder('ShopNotifications')->child('NewOrders');
        $NewOrdersNotification->deleteObject($Order->get('OriginalOrder'));
        $NewOrdersNotification->addObject($Order);
    }

    # only for unregistered users : update Customer billing address on using the editable order as new order
    # this is needed for having the correct billing address on the customer
    # if we change the address on the editable order and delete the order
    # test if we have an anonymous customer and update the address on the customer
    my $Customer = $Order->get('Customer');
    unless( scalar(@{$Customer->get('Users')}) ){

        my $BillingAddress = $Order->get('BillingAddress');
        my $AddressFolder = $Customer->folder('Addresses');
        my $NewAlias = $AddressFolder->nextAlias('BillingAddress %s');
        my $NewBillingAddress = $BillingAddress->clone({ 'Alias' => $NewAlias, 'Parent' => $AddressFolder });

        my $CustomerBillingAddress = $Customer->get('BillingAddress');
        # set cloned address
        $Customer->set({'BillingAddress' => $NewBillingAddress});
        # delete old address
        $CustomerBillingAddress->delete;
    }
}

#========================================================================================
# function     OnEditOrder2OrderSendConfirmationMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnEditOrder2OrderSendConfirmationMail($Servlet, $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by hook UI_EditOrder2OrderSendConfirmationMail.
#               Sends the order confirmation e-mail to the customer if the option
#               $hParams->{Options}->{SendConfirmationMail} are set.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Order - order object</li>
#                   <li>Options - options hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnEditOrder2OrderSendConfirmationMail {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};
    return unless $hParams->{Options}->{NewCreatedOrder};

    my $MailType = GetShopMailType( $Order->getSite(), 'CustomerOrderConfirmation', 1 );
    return if not defined $MailType;

    eval {SendOrderConfirmationMail($hParams->{'Order'});};

    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);

        ResetError();
    }
    return ;
}

1;
