use strict;
use warnings;
use Test::More tests => 32;

use Carp qw ( croak );
use DE_EPAGES::Core::API::File qw ( DeleteFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw (
    ConnectTestSite
    DisconnectSite
    GetCurrentDBHandle
);
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadClassByAlias
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::PriceList::API::CSV::PriceListExportImport;
use List::Util qw ( first );
use Readonly;
use Test::Deep;
use Text::CSV_XS;

Readonly::Scalar my $ROOTPATH         => 'Test';
Readonly::Scalar my $SHOP             => 'TestShop';
Readonly::Scalar my $CSV_FILENAME     => 'discounts.csv';
Readonly::Scalar my $TMP_FILENAME     => 'tmp.csv';
Readonly::Scalar my $NEW_DISCOUNT     => 100;
Readonly::Hash   my %hExpectedResults => (
    # product alias       => expected product path after CSV import
    'be_4/04/01'          => '"be_4/04/01"',
    'Shirt'               => 'Shirt',
    'be_404/0104/004'     => '"be_4/04/01"/SubProducts/"be_404/0104/004"',
    'be_4040104005'       => '"be_4/04/01"/SubProducts/be_4040104005',
    'cg_0100//50/40/ /01' => '"cg_0100//50/40/ /01"',
    'FancyShirt / 1'      => 'FancyShirt/SubProducts/"FancyShirt / 1"',
    'mlaut hirt'        => 'FancyShirt/SubProducts/"mlaut hirt"',
);
Readonly::Array my @aExpectedPriceLists => (
    'RegularCustomer_EUR',
    '"Regular/PriceList"',
    "\"Li\x{df}t with \x{dc}ml\x{e4}uts\"",
);

sub TestExportImport {
    my $Shop = LoadObjectByPath( JoinObjectPath([ $ROOTPATH, $SHOP ]) );
    my $ProductsPath = join('/', $ROOTPATH, $SHOP, 'Products');

    # ExporterImporter options
    my $ExportImporter = DE_EPAGES::PriceList::API::CSV::PriceListExportImport->new;
    my $hFileFormat = {
        'LanguageCode2s' => [],
    };

    my $PriceLists = $Shop->get('PriceLists');
    foreach my $Alias ( keys %hExpectedResults ) {
        # csvExport using PriceListExportDriver
        my $ProductAlias = $hExpectedResults{$Alias};
        my $Product = LoadObjectByPath("$ProductsPath/$ProductAlias");  ## no critic qw(ProhibitPathConcatenation)

        SKIP: {
            skip ('Product has missing attribute HasSubOwnPrices set to \'1\'.)', 2)
                if ( $Product->get('SuperProduct') && !$Product->get('HasSubOwnPrices') );

            $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is deleted during csvExport
            $ExportImporter->csvExport($Shop, $CSV_FILENAME, $hFileFormat, [ $Product->id ]);

            # csvImport using an own handler (cf. below)
            $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is deleted during csvExport
            my $Importer = DE_EPAGES::PriceList::t::MyImportHandler->importCSV($CSV_FILENAME, $hFileFormat);
            is( $Importer->{'Items'}->[0]->{'Product'}, $hExpectedResults{$Alias}, "test '$Alias'");

            # modify some CSV values
            my $CSV = Text::CSV_XS->new({ $Importer->converter->formatSpec })
                or croak('Cannot use CSV: ' . Text::CSV->error_diag ());

            my $IsFirstLine = 1;
            open (my $CSVData, '<', $CSV_FILENAME) or croak "Could not open '$CSV_FILENAME' $!\n";
            open (my $NewData, '>', $TMP_FILENAME) or croak "Could not open '$TMP_FILENAME' $!\n";
            while ( my $Line = <$CSVData> ) {
                chomp $Line;
                if ( $CSV->parse($Line) ) {
                    my @Fields = $CSV->fields();
                    if ($IsFirstLine == 1) {
                        $CSV->print($NewData, \@Fields);
                        $IsFirstLine = 0;
                        next;
                    }
                    $Fields[4] = $NEW_DISCOUNT;
                    $CSV->print($NewData, \@Fields);
                } else {
                    warn "Line could not be parsed: $Line\n";
                }
            }
            close ($CSVData) or warn $! ? "Error closing $CSVData: $!" : "Exit status $?";
            close ($NewData) or warn $! ? "Error closing $NewData: $!" : "Exit status $?";

            # csvImport into database
            $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is delete during csvExport
            $ExportImporter->csvImport($Shop, $TMP_FILENAME, $hFileFormat);

            # verify new discounts set to $NEW_DISCOUNT
            my $Index = first { $PriceLists->[$_]->alias eq 'RegularCustomer_EUR' } 0 .. (scalar @{ $PriceLists });
            my $aQuantityDiscounts = $PriceLists->[$Index]->getQuantityDiscounts( $Product );
            is( $aQuantityDiscounts->[$Index]->{'Discount'}, $NEW_DISCOUNT, "new discount value for '$Alias' is set" );
        }
    }

    # Cleanup
    DeleteFile($CSV_FILENAME);
    DeleteFile($TMP_FILENAME);

    return;
}

sub TestExportImportOfPriceLists {
    my $Shop = LoadObjectByPath( JoinObjectPath([ $ROOTPATH, $SHOP ]) );

    my $ProductPath = join('/', $ROOTPATH, $SHOP, 'Products', 'Shirt');
    my $Product = LoadObjectByPath($ProductPath);

    my $hFileFormat = {
        'LanguageCode2s' => [],
        'LocaleID'       => $Shop->get('LocaleID'),
    };
    # export discounts for 'Shirt'
    my $ExportImporter = DE_EPAGES::PriceList::API::CSV::PriceListExportImport->new;
    $ExportImporter->csvExport($Shop, $CSV_FILENAME, $hFileFormat, [ $Product->id ]);

    # import discounts for 'Shirt'
    $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');  # LocaleID is delete during csvExport
    my $Importer = DE_EPAGES::PriceList::t::MyImportHandler->importCSV($CSV_FILENAME, $hFileFormat);

    # compare imported price lists with expected price lists
    my @ImportedPriceLists = map { $_->{'PriceList'} }@{ $Importer->{'Items'} };
    cmp_bag( \@ImportedPriceLists, \@aExpectedPriceLists,
        'all pricelists were imported successfully for \'' . $Product->alias . '\'');
    return;
}

sub TestCSVImport {
    my $Shop = LoadObjectByPath(JoinObjectPath([ $ROOTPATH, $SHOP ]));

    # (1) Define a new pricelist
    my $TaxModel = 0;
    my $PriceListFolder = $Shop->folder('PriceLists');
    my $PriceList = LoadClassByAlias('PriceList')->insertObject({
        Alias        => 'NewPriceList',
        Parent       => $PriceListFolder,
        CurrencyID   => 'EUR',
    });


    # (2) Create a new product and assign some discounts
    my $Product = $Shop->get('ProductFolder')->insertChild( $Shop->get('ProductClass'), {
        'Alias' => 'TestProduct'
    });


    # (3) ScalePrice
    my $ahScalePrices = [
        {
            Quantity => 1,
            Price    => 10,
        },
        {
            Quantity => 10,
            Price    => 100,
        },
    ];
    $PriceList->setScalePrices($Product, $TaxModel, $ahScalePrices);
    _SharedCSVImportRoutine($Shop, $Product, 'scaleprice', 'price', $ahScalePrices, sub {
        my ($Line) = @_;
        my @Columns = split(/[,;]/, $Line);
        return unless $Columns[2] && $Columns[3]; ## It's a scale price
        $Columns[3] = 1;
        return join(',', @Columns);
    });


    # (4) QuantityDiscount
    my $ahQuantityDiscounts = [
        {
            Quantity => 30,
            Discount => 5,
        },
        {
            Quantity => 40,
            Discount => 15,
        },
    ];
    $PriceList->setQuantityDiscounts($Product, $TaxModel, $ahQuantityDiscounts);
    _SharedCSVImportRoutine($Shop, $Product, 'quantitydiscount', 'discount', $ahQuantityDiscounts, sub {
        my ($Line) = @_;
        my @Columns = split(/[,;]/, $Line);
        return unless $Columns[2] && $Columns[4]; ## It's a quantity discount
        $Columns[4] = 1;
        return join(',', @Columns);
    });


    # (5) ValueDiscount
    my $ahValueDiscounts = [
        {
            Value    => 100,
            Discount => 5,
        },
        {
            Value    => 200,
            Discount => 15,
        },
    ];
    $PriceList->setValueDiscounts($Product, $TaxModel, $ahValueDiscounts);
    _SharedCSVImportRoutine($Shop, $Product, 'valuediscount', 'discount', $ahValueDiscounts, sub {
        my ($Line) = @_;
        my @Columns = split(/[,;]/, $Line);
        return unless $Columns[5] && $Columns[6]; ## It's a value discount
        $Columns[6] = "1\n";
        return join(',', @Columns);
    });

    return;
}

sub _SharedCSVImportRoutine {
    my ($Shop, $Product, $SqlTable, $SqlColumn, $ahDiscounts, $subModifyDiscount) = @_;

    my $dbi = GetCurrentDBHandle();

    # (3) Validate the amount of discounts written to the database
    #     --> Expected: number of inserted product discounts equals length of $ahScalePrices
    my $SQL = "SELECT count(*) FROM $SqlTable WHERE productid = ?";
    my $aRowCount = $dbi->execute($SQL, [ $Product->id ]);
    is( $aRowCount->[0]->[0], scalar @{$ahDiscounts},
        "$SqlTable: discounts were written to the table '$SqlTable'");


    # (4) Export and re-import the same discounts, again
    my $hFileFormat = {
        'LanguageCode2s' => [],
        'LocaleID'       => $Shop->get('LocaleID'),
    };
    my $ExporterImporter = DE_EPAGES::PriceList::API::CSV::PriceListExportImport->new;
    $ExporterImporter->csvExport($Shop, $CSV_FILENAME, $hFileFormat, [ $Product->id ]);

    $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');
    $ExporterImporter->csvImport($Shop, $CSV_FILENAME, $hFileFormat);


    # (6) Validate discounts
    #     --> Expected: no new rows were added to the database, since the discounts haven't changed
    $aRowCount = $dbi->execute($SQL, [ $Product->id ]);
    is( $aRowCount->[0]->[0], scalar @{$ahDiscounts},
        "$SqlTable: only the previous imported discounts are stored in the database");


    # (7) Modify the price of the discounts to 1 EUR
    my $DISCOUNTS_FNAME = 'tmp.csv';
    open(my $DC, '>', $DISCOUNTS_FNAME) or die "Could not open file $!";
    open(my $FH, '<', $CSV_FILENAME)    or die "Could not open file $!";
    while (<$FH>) {
        unless ($_ =~ /^TestProduct/) {
            print $DC $_;
            next;
        }
        my $Line = $subModifyDiscount->($_);
        next unless defined $Line;
        print $DC $Line;
    }
    close $FH or die "Could not close file $!";
    close $DC or die "Could not close file $!";


    # (8) Import product discounts, again
    $hFileFormat->{'LocaleID'} = $Shop->get('LocaleID');
    $ExporterImporter->csvImport($Shop, $DISCOUNTS_FNAME, $hFileFormat);


    # (9) Re-import discounts
    #     --> Expected: rows are just updated, no new rows were added to the database
    $SQL = "SELECT $SqlColumn FROM $SqlTable WHERE productid = ?";
    my $aaRows = $dbi->execute($SQL, [ $Product->id ]);
    is( scalar @{$aaRows}, scalar @{$ahDiscounts},
        "$SqlTable: still no further rows were added to the database");

    if ($SqlTable eq 'scaleprice') {
        is ($_->[0], '0.8620', "$SqlTable: new discount price is 0.86 EUR") foreach (@{$aaRows});
    } else {
        is ($_->[0], '1', "$SqlTable: new discount value equals 1") foreach (@{$aaRows});
    }

    return;
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    DeleteFile($CSV_FILENAME);
    DeleteFile($TMP_FILENAME);

    return 1;
}

sub Init {
    Cleanup();

    my $System = LoadRootObject();
    $System->insertChild('Object', {
        Alias => 'Test'
    });
    ImportObjects('import.xml', $ROOTPATH);

    return 1;
}

RunScript(Debug => 1, Sub => sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    TestExportImportOfPriceLists();
    TestExportImport();
    TestCSVImport();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});


#========================================================================================
# package      DE_EPAGES::PriceList::t::MyImportHandler;
# base         DE_EPAGES::Core::API::CSV::BaseImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a simple handler to read CSV files.
#========================================================================================
package DE_EPAGES::PriceList::t::MyImportHandler;
use base qw ( DE_EPAGES::Core::API::CSV::BaseImportHandler );

sub new {  ## no critic qw(RequireArgUnpacking)
    my $class = shift;
    my $self = $class->SUPER::new(@_);

    $self->{'Items'} = [];

    return $self;
}

sub importLine {
    my $self = shift;

    my @Header = map { /\[(.*?)\]/ } @{ $self->header };
    my %hColumns;
    @hColumns{ @Header } = @{ $self->topID('Item') };
    push @{ $self->{'Items'} }, \%hColumns;

    return;
}

1;
